#!/bin/bash
# Copyright 2019 Michal Žejdl
# read the dot1dStp from BRIDGE-MIB and store it to mysql tables dot1dStp*

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
source $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
source $functions

address=$1
agent=$2

errlog=$logdir/snmp/$(basename $0 .sh).err

# a new file descriptor to prevent pipe from closing
exec 3>$mysqlwpipe

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# 2019-01-10 - recent MikroTik RouterOS returns only .1.0 and .2.0
# so therefore 2>/dev/null to spare error log file
result=($(snmpget -m '' -v 1 -c "$snmprcommunity" -Onqt "$address" \
.1.3.6.1.2.1.17.2.1.0 \
.1.3.6.1.2.1.17.2.2.0 \
.1.3.6.1.2.1.17.2.3.0 \
.1.3.6.1.2.1.17.2.4.0 \
.1.3.6.1.2.1.17.2.5.0 \
.1.3.6.1.2.1.17.2.6.0 \
.1.3.6.1.2.1.17.2.7.0 \
.1.3.6.1.2.1.17.2.8.0 \
.1.3.6.1.2.1.17.2.9.0 \
.1.3.6.1.2.1.17.2.10.0 \
.1.3.6.1.2.1.17.2.11.0 \
.1.3.6.1.2.1.17.2.12.0 \
.1.3.6.1.2.1.17.2.13.0 \
.1.3.6.1.2.1.17.2.14.0 \
2>/dev/null \
))

snmp_status=$?

# by my observation status 1 means no returned values at all so leave now
#	and don't bother with parsing $result
if test $snmp_status -eq 1
then
	echo "$(date '+%F %T') $(basename $0) address $address agent $agent snmp_status $snmp_status" >>$errlog
	exit $snmp_status
fi

IFS=$oldifs

# prepare for inserting into myslq after incomplete response
dot1dStpProtocolSpecification=null
dot1dStpPriority=null
dot1dStpTimeSinceTopologyChange=null
dot1dStpTopChanges=null
dot1dStpRootCost=null
dot1dStpRootPort=null
dot1dStpMaxAge=null
dot1dStpHelloTime=null
dot1dStpHoldTime=null
dot1dStpForwardDelay=null
dot1dStpBridgeMaxAge=null
dot1dStpBridgeHelloTime=null
dot1dStpBridgeForwardDelay=null

for row in "${result[@]}"
do
	oid=${row%% *}
	value=${row#* }

	case "$oid" in
	.1.3.6.1.2.1.17.2.5.0)
		# remove the quotes around the string
		value=${value#\"}
		value=${value%\"}
		# sanitize the string
		replace_specchars "$value" && value=$without_specchars
		;;
	*)
		is_natural_number "$value" || value=null
	esac

	case "$oid" in
	.1.3.6.1.2.1.17.2.1.0)
		dot1dStpProtocolSpecification=$value
		;;
	.1.3.6.1.2.1.17.2.2.0)
		dot1dStpPriority=$value
		;;
	.1.3.6.1.2.1.17.2.3.0)
		dot1dStpTimeSinceTopologyChange=$value
		;;
	.1.3.6.1.2.1.17.2.4.0)
		dot1dStpTopChanges=$value
		;;
	.1.3.6.1.2.1.17.2.5.0)
		dot1dStpDesignatedRoot=$value
		;;
	.1.3.6.1.2.1.17.2.6.0)
		dot1dStpRootCost=$value
		;;
	.1.3.6.1.2.1.17.2.7.0)
		dot1dStpRootPort=$value
		;;
	.1.3.6.1.2.1.17.2.8.0)
		dot1dStpMaxAge=$value
		;;
	.1.3.6.1.2.1.17.2.9.0)
		dot1dStpHelloTime=$value
		;;
	.1.3.6.1.2.1.17.2.10.0)
		dot1dStpHoldTime=$value
		;;
	.1.3.6.1.2.1.17.2.11.0)
		dot1dStpForwardDelay=$value
		;;
	.1.3.6.1.2.1.17.2.12.0)
		dot1dStpBridgeMaxAge=$value
		;;
	.1.3.6.1.2.1.17.2.13.0)
		dot1dStpBridgeHelloTime=$value
		;;
	.1.3.6.1.2.1.17.2.14.0)
		dot1dStpBridgeForwardDelay=$value
		;;
	esac
done

echo "insert dot1dStp (agent, time, \
dot1dStpProtocolSpecification, \
dot1dStpPriority, \
dot1dStpTimeSinceTopologyChange, \
dot1dStpTopChanges, \
dot1dStpDesignatedRoot, \
dot1dStpRootCost, \
dot1dStpRootPort, \
dot1dStpMaxAge, \
dot1dStpHelloTime, \
dot1dStpHoldTime, \
dot1dStpForwardDelay, \
dot1dStpBridgeMaxAge, \
dot1dStpBridgeHelloTime, \
dot1dStpBridgeForwardDelay \
) values ($agent, unix_timestamp(), \
$dot1dStpProtocolSpecification, \
$dot1dStpPriority, \
$dot1dStpTimeSinceTopologyChange, \
$dot1dStpTopChanges, \
'$dot1dStpDesignatedRoot', \
$dot1dStpRootCost, \
$dot1dStpRootPort, \
$dot1dStpMaxAge, \
$dot1dStpHelloTime, \
$dot1dStpHoldTime, \
$dot1dStpForwardDelay, \
$dot1dStpBridgeMaxAge, \
$dot1dStpBridgeHelloTime, \
$dot1dStpBridgeForwardDelay \
) on duplicate key update \
time = unix_timestamp(), \
dot1dStpProtocolSpecification = $dot1dStpProtocolSpecification, \
dot1dStpPriority = $dot1dStpPriority, \
dot1dStpTimeSinceTopologyChange = $dot1dStpTimeSinceTopologyChange, \
dot1dStpTopChanges = $dot1dStpTopChanges, \
dot1dStpDesignatedRoot = '$dot1dStpDesignatedRoot', \
dot1dStpRootCost = $dot1dStpRootCost, \
dot1dStpRootPort = $dot1dStpRootPort, \
dot1dStpMaxAge = $dot1dStpMaxAge, \
dot1dStpHelloTime = $dot1dStpHelloTime, \
dot1dStpHoldTime = $dot1dStpHoldTime, \
dot1dStpForwardDelay = $dot1dStpForwardDelay, \
dot1dStpBridgeMaxAge = $dot1dStpBridgeMaxAge, \
dot1dStpBridgeHelloTime = $dot1dStpBridgeHelloTime, \
dot1dStpBridgeForwardDelay = $dot1dStpBridgeForwardDelay \
;" >&3


# dot1dStpPortTable

# store port numbers into an array
dot1dStpPort=($(snmpwalk -Cc -m '' -v 1 -c "$snmprcommunity" -Oqv "$address" \
.1.3.6.1.2.1.17.2.15.1.1 \
))

for port in "${dot1dStpPort[@]}"
do
	is_natural_number "$port" || continue

	# prepare an array of oids for the current $port
	# (dot1dStpPortPriority..dot1dStpPortForwardTransitions)
	for oid in {2..10}
	do
		oids[$oid]=.1.3.6.1.2.1.17.2.15.1."$oid"."$port"
	done

	oldifs=$IFS
	# one line = one array element; and focus only on the first line
	IFS=$'\n'
	result=($(snmpget -m '' -v 1 -c "$snmprcommunity" -Onq "$address" \
				"${oids[@]}"))
	IFS=$oldifs

	# prepare for inserting into myslq after incomplete response
	dot1dStpPortPriority=null
	dot1dStpPortState=null
	dot1dStpPortEnable=null
	dot1dStpPortPathCost=null
	dot1dStpPortDesignatedRoot=""
	dot1dStpPortDesignatedCost=null
	dot1dStpPortDesignatedBridge=""
	dot1dStpPortDesignatedPort=""
	dot1dStpPortForwardTransitions=null
	dot1dStpPortPathCost32=null

	for row in "${result[@]}"
	do
		oid=${row%% *}
		value=${row#* }

		case "$oid" in
		.1.3.6.1.2.1.17.2.15.1.6."$port" \
			|.1.3.6.1.2.1.17.2.15.1.8."$port" \
			|.1.3.6.1.2.1.17.2.15.1.9."$port")

			# remove the quotes around the string
			value=${value#\"}
			value=${value%\"}
			# sanitize the string
			replace_specchars "$value" && value=$without_specchars
			;;
		*)
			is_natural_number "$value" || value=null
		esac

		case "$oid" in
		.1.3.6.1.2.1.17.2.15.1.2."$port")
			dot1dStpPortPriority=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.3."$port")
			dot1dStpPortState=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.4."$port")
			dot1dStpPortEnable=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.5."$port")
			dot1dStpPortPathCost=$value

			# see BRIDGE-MIB.txt
			if test "$dot1dStpPortPathCost" -eq 65535
			then
				value=$(snmpget -m '' -v 1 \
						-c "$snmprcommunity" -Oqv \
						"$address" \
						.1.3.6.1.2.1.17.2.15.1.11."$port" \
						2>/dev/null)
				is_natural_number "$value" \
					&& dot1dStpPortPathCost32=$value
			fi
			;;
		.1.3.6.1.2.1.17.2.15.1.6."$port")
			dot1dStpPortDesignatedRoot=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.7."$port")
			dot1dStpPortDesignatedCost=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.8."$port")
			dot1dStpPortDesignatedBridge=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.9."$port")
			dot1dStpPortDesignatedPort=$value
			;;
		.1.3.6.1.2.1.17.2.15.1.10."$port")
			dot1dStpPortForwardTransitions=$value
			;;
		esac
	done

	echo "insert dot1dStpPort (agent, time, \
dot1dStpPort, \
dot1dStpPortPriority, \
dot1dStpPortState, \
dot1dStpPortEnable, \
dot1dStpPortPathCost, \
dot1dStpPortDesignatedRoot, \
dot1dStpPortDesignatedCost, \
dot1dStpPortDesignatedBridge, \
dot1dStpPortDesignatedPort, \
dot1dStpPortForwardTransitions, \
dot1dStpPortPathCost32 \
) values ($agent, unix_timestamp(), \
$port, \
$dot1dStpPortPriority, \
$dot1dStpPortState, \
$dot1dStpPortEnable, \
$dot1dStpPortPathCost, \
'$dot1dStpPortDesignatedRoot', \
$dot1dStpPortDesignatedCost, \
'$dot1dStpPortDesignatedBridge', \
'$dot1dStpPortDesignatedPort', \
$dot1dStpPortForwardTransitions, \
$dot1dStpPortPathCost32 \
) on duplicate key update \
time = unix_timestamp(), \
dot1dStpPortPriority = $dot1dStpPortPriority, \
dot1dStpPortState = $dot1dStpPortState, \
dot1dStpPortEnable = $dot1dStpPortEnable, \
dot1dStpPortPathCost = $dot1dStpPortPathCost, \
dot1dStpPortDesignatedRoot = '$dot1dStpPortDesignatedRoot', \
dot1dStpPortDesignatedCost = $dot1dStpPortDesignatedCost, \
dot1dStpPortDesignatedBridge = '$dot1dStpPortDesignatedBridge', \
dot1dStpPortDesignatedPort = '$dot1dStpPortDesignatedPort', \
dot1dStpPortForwardTransitions = $dot1dStpPortForwardTransitions, \
dot1dStpPortPathCost32 = $dot1dStpPortPathCost32 \
;" >&3

done
