#!/bin/bash
# Copyright 2018 Michal Žejdl
# get and store some columns of entityPhysicalTable to the SQL table

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

oid_entPhysicalEntry=.1.3.6.1.2.1.47.1.1.1.1
oid_entPhysicalDescr=$oid_entPhysicalEntry.2
oid_entPhysicalClass=$oid_entPhysicalEntry.5
oid_entPhysicalFirmwareRev=$oid_entPhysicalEntry.9
oid_entPhysicalSoftwareRev=$oid_entPhysicalEntry.10

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# get the indexes by snmpwalk entPhysicalDescr
# because some switches from Cisco, Planet, 3Com return no entPhysicalIndex

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'

# no retry (-r 0)
descriptions=($(snmpwalk -Cc -m '' -v 1 -r 0 -c $snmprcommunity -Onq $address \
$oid_entPhysicalDescr \
))

snmp_status=$?
test $snmp_status -eq 0 || exit $snmp_status

IFS=$oldifs

for ((desc_row = 0; desc_row < ${#descriptions[@]}; desc_row++))
do
	oid=${descriptions[desc_row]%% *}
	value=${descriptions[desc_row]#* }

	oid_base=${oid%.*}
	oid_index=${oid##*.}

	# test if row contain entPhysicalDescr OID
	test "$oid_base" = "$oid_entPhysicalDescr" || continue

	# check validity of the parsed index
	is_natural_number "$oid_index" || continue

	# remove the quotes around the string
	entPhysicalDescr=${value#\"}
	entPhysicalDescr=${entPhysicalDescr%\"}

	# sanitize the string
	replace_specchars "$entPhysicalDescr" \
		&& entPhysicalDescr=$without_specchars
	
	# write some output
	#echo "$(date "+%F %T") $(basename $0): agent $agent, address $address entPhysicalIndex $oid_index, entPhysicalDescr $entPhysicalDescr"

	oldifs=$IFS
	IFS=$'\n'
	other_oids=($(snmpget -m '' -v 1 -r 0 -c $snmprcommunity -Cf -Onq \
$address \
$oid_entPhysicalClass.$oid_index \
$oid_entPhysicalFirmwareRev.$oid_index \
$oid_entPhysicalSoftwareRev.$oid_index))
	snmp_status=$?
	IFS=$oldifs

	entPhysicalClass=0
	entPhysicalFirmwareRev=""
	entPhysicalSoftwareRev=""

	for ((oid_row = 0; oid_row < ${#other_oids[@]}; oid_row++))
	do
		case ${other_oids[oid_row]%% *} in
		$oid_entPhysicalClass.$oid_index)
			entPhysicalClass=${other_oids[$oid_row]#* }
			is_natural_number "$entPhysicalClass" \
				|| entPhysicalClass=0
			;;
		$oid_entPhysicalFirmwareRev.$oid_index)
			entPhysicalFirmwareRev=${other_oids[$oid_row]#* \"}
			entPhysicalFirmwareRev=${entPhysicalFirmwareRev%\"}
			replace_specchars "$entPhysicalFirmwareRev" \
				&& entPhysicalFirmwareRev=$without_specchars
			;;
		$oid_entPhysicalSoftwareRev.$oid_index)
			entPhysicalSoftwareRev=${other_oids[$oid_row]#* \"}
			entPhysicalSoftwareRev=${entPhysicalSoftwareRev%\"}
			replace_specchars "$entPhysicalSoftwareRev" \
				&& entPhysicalSoftwareRev=$without_specchars
			;;
		esac
	done

	# finaly write the variables into the db
	echo "insert entPhysical (\
agent, \
time, \
entPhysicalIndex, \
entPhysicalDescr, \
entPhysicalClass, \
entPhysicalFirmwareRev, \
entPhysicalSoftwareRev) \
values (\
$agent, \
unix_timestamp(), \
$oid_index, \
'$entPhysicalDescr', \
$entPhysicalClass, \
'$entPhysicalFirmwareRev', \
'$entPhysicalSoftwareRev') \
on duplicate key update \
time = unix_timestamp(), \
entPhysicalDescr = '$entPhysicalDescr', \
entPhysicalClass = $entPhysicalClass, \
entPhysicalFirmwareRev = '$entPhysicalFirmwareRev', \
entPhysicalSoftwareRev = '$entPhysicalSoftwareRev';\
" >&3

done

exit $snmp_status
