#!/bin/bash
# Copyright 2009, 2010, 2017 Michal Žejdl
# count MAC adresses from arp or nd table which were alive for the last $hours
# $prefix is an optional argument for IPv4 or IPv6 addresses narrowing
# for net-snmp -> NetMRG graph
# dependence: mysql

confdir=$(dirname $0)/..
config=$confdir/defaults.sh
test -f $config || exit
. $config

print_usage_and_exit ()
{
	echo "usage: $(basename $0) <arp|nd> <hours> [prefix]"
	exit
}

if test $# -lt 2; then
	print_usage_and_exit
fi

table=$1
hours=$2

if test $# -ge 3; then
	prefix=$3
	case "$table" in
	arp)
		column=ip
		;;
	nd)
		column=ip6
		;;
	esac
	and_condition="and $column like '$prefix%'"
fi

case "$table" in
arp|nd)
	;;
*)
	print_usage_and_exit
	;;
esac

# ! before test and 2> are for non numerical $hours value
if ! test "$hours" -gt 0 2>/dev/null; then
	echo "<hours> must be a positive integer"
	exit
fi

mysql -e "select count(distinct(mac)) from $table where time >= unix_timestamp() - $hours * 3600 $and_condition" $select_opts
