#!/bin/bash
# Copyright 2010 - 2018 Michal Žejdl
# pro SNMP pozitivní volá collector.sh tak, aby stihl všechny v daném intervalu

confdir=$(dirname $0)/../../core
config=$confdir/defaults.sh
test -f $config || exit
. $config

exec 3> $mysqlwpipe

snmp=$netldbdir/snmp

# minimální doba na usnutí v cyklu
minusleep=1

# počáteční nastavení spaní
# počet všech s DNS, živých a s SNMP
if ! kandidatu=$(mysql -e "select count(*) from checked where dns != '' and live = 'y' and snmp = 'y'" $modify_opts -Ns); then kandidatu=0; fi
# počet skriptů, které se (teoreticky) spouští pro každý prvek
skriptu=$(find $snmp -type f -name "*.sh" | wc -l)
# výpočet počáteční doby spaní - jednička je záchrana pro dělení
delay=$((1000000 * $snmpinterval / ($kandidatu * $skriptu + 1)))

while true; do
  	# stop this golem if "šém" is removed
	test -e $rundir/$(basename $0 .sh).sem || exit
  logdate=$(date +'%F %T')

  # dobýváme se do db, dokud nedostaneme odpověď
  # hledáme nejstarší se jménem, živý, s SNMP, který ještě není zkoumán
  # a který už nebyl čtený delší (nebo stejnou) dobu, než je interval
  while ! prvek=$(mysql -e "select dns from checked where dns != '' and live = 'y' and snmp = 'y' and collecting = 'n' and lastsnmpread <= unix_timestamp() - $snmpinterval order by lastsnmpread limit 1" $modify_opts -Ns); do
    echo "$logdate nějaký problém s db, čekám"
    sleep 10
  done

  # pokud řetězec není prázdný
  if test $prvek; then
    # raději se zasekneme na chybě
    while ! mysql -e "update checked set collecting = 'y' where dns = '$prvek'" $modify_opts; do sleep 10; done
    # tohle jednou nezafungovalo včas (pomalý flush v rouře při vysoké zátěži?)
    # a nad sw4210-dj se spustily desítky procesů collector.sh
    # přepínač to neustál, a pak odpovídal na snmpwalk pomalu a nekonečně
    # takže to z nedostatku lepších nápadů zatím vypnu
    #echo "update checked set collecting = 'y' where dns = '$prvek';" >&3
    echo "$logdate čtu SNMP z prvku $prvek"
    $snmp/collector.sh $prvek &
    # bylo co zpracovávat, zkrátíme dobu na spaní, pokud to ještě jde
    if test $delay -gt $minusleep; then ((delay--)); fi
  else
    # není co dělat, pospíme si déle
    ((delay++))
  fi

	# make the sleep interruptible
	seconds=$((delay / 1000000))
	for ((i = 0; i < $seconds; i++))
	do
		# stop this golem if "šém" is removed
		test -e $rundir/$(basename $0 .sh).sem || exit
		sleep 1
	done

	# sleep the rest
	usleep $((delay % 1000000))
done
